/***************************************************************************
                          dspushbutton.cpp  -  description
                             -------------------
    begin                : Don Okt 30 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dspushbutton.h"

#include <kapplication.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmimetype.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kurl.h>
#include <krun.h>
#include <qdragobject.h>
#include <qmime.h>
#include <qtooltip.h>
#include <qevent.h>

DSPushButton::DSPushButton(QWidget *parent, const char *name )
    : KPushButton(parent,name),
      DCOPObject( "DragStacK" )
{
    updateIcon();
    connect( this, SIGNAL( clicked() ), this, SLOT( run() ) );
    setAcceptDrops( true );
}

DSPushButton::~DSPushButton()
{
}

void DSPushButton::dragEnterEvent(QDragEnterEvent* event)
{
    event->accept( QUriDrag::canDecode(event) );
}

void DSPushButton::dropEvent(QDropEvent* event)
{
    QStringList list;
    
    if ( QUriDrag::decodeToUnicodeUris(event, list) ) {
        files.push( list );
        updateIcon();
    }
}

void DSPushButton::mousePressEvent( QMouseEvent* e )
{
    presspos = e->pos();
    pressed = ( e->button() == Qt::LeftButton );
    if( e->button() == Qt::RightButton ) {
        KPopupMenu* mnu = new KPopupMenu( this );
        mnu->insertTitle( i18n("DragStacK") );
        if( !files.isEmpty() ) {
            KPopupMenu* mnuItem = new KPopupMenu( mnu );
            connect( mnuItem, SIGNAL( activated(int) ), this, SLOT( top(int) ) );
            for( int i = (signed int)files.count() - 1; i >= 0; i-- )
                mnuItem->insertItem( (files[i])[0], i + 1000 );

            mnu->insertItem( i18n("&Clear Stack"), this, SLOT( clear() ) );
            mnu->insertItem( SmallIconSet( "edittrash" ), i18n("&Remove Item"), this, SLOT( remove() ) );
            mnu->insertSeparator();
            mnu->insertItem( i18n("Select Item"), mnuItem );
            mnu->insertSeparator();
            mnu->insertItem( SmallIconSet( "fileopen" ), i18n("&Open"), this, SLOT( run() ) );
            mnu->insertSeparator();
        }
        mnu->insertItem( SmallIconSet( "dragstack" ), i18n("&About..."), this, SIGNAL( about() ) );
        mnu->popup( e->globalPos() );
    }
}

void DSPushButton::mouseMoveEvent( QMouseEvent* e )
{
    if ( !files.isEmpty() && pressed && ( presspos - e->pos() ).manhattanLength() > KApplication::startDragDistance() ) {
        QUriDrag* drag = new QUriDrag( this );

        drag->setUnicodeUris( files.pop() );
        drag->drag();
        pressed = false;
    }
}

void DSPushButton::mouseReleaseEvent( QMouseEvent* )
{
    updateIcon();
    pressed = false;
}

void DSPushButton::updateIcon()
{
    if( !files.isEmpty() ) {
        QStringList list = files.last();
        setPixmap( kapp->iconLoader()->loadIcon( KMimeType::iconForURL( KURL( list[0] ) ), KIcon::Panel ) );

        QString tt;
        for( unsigned int i = 0; i < list.count(); i++ )
            tt.append( list[i] + "\n" );

        QToolTip::remove( this );
        QToolTip::add( this, tt );
    } else {
        QPixmap pix = kapp->iconLoader()->loadIcon( "empty", KIcon::Panel );
        setPixmap( pix );
        QToolTip::remove( this );
        QToolTip::add( this, QString::null );
    }
}

void DSPushButton::clear()
{
    files.clear();
    updateIcon();
}

void DSPushButton::remove()
{
    files.remove( files.last() );
    updateIcon();
}

int DSPushButton::count() const
{
    return files.count();
}

void DSPushButton::push( KURL url )
{
    if( url.isValid() ) {
        files.push( QStringList( url.path() ) );
        updateIcon();
    }
}

KURL DSPushButton::pop()
{
    QStringList list = files.pop();
    updateIcon();

    return KURL( list[0] );
}

void DSPushButton::run()
{
    if( !files.isEmpty() ) {
        KRun* run = new KRun( files.last().first() );
        run->setAutoDelete( true );
    }
}

void DSPushButton::top( int i )
{
    QStringList list = files[i-1000];
    files.remove( files.at( i-1000 ) );
    files.push( list );
    updateIcon();
}
